/* event.c - a Ruby event class for Corporate Time
 * 
 * $Id: event.c,v 1.6 2002/09/25 09:27:53 ianmacd Exp $
 *
 * Author: Ian Macdonald <ian@caliban.org>
 *
 * Copyright (C) 2002 Ian Macdonald
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software Foundation,
 *   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <ruby.h>


VALUE cEvent;

/* Event.initialize */
VALUE ev_init(VALUE self, VALUE args)
{
    int length;

    rb_iv_set(self, "@uid", Qnil);
    rb_iv_set(self, "@start", Qnil);
    rb_iv_set(self, "@duration", Qnil);
    rb_iv_set(self, "@summary", Qnil);
    rb_iv_set(self, "@location", Qnil);
    rb_iv_set(self, "@description", Qnil);

    length = RARRAY(args)->len;

    while (length-- > 0) {
	switch (length) {
	case 0:
	    rb_iv_set(self, "@start", rb_ary_pop(args));
	    break;
	case 1:
	    rb_iv_set(self, "@duration", rb_ary_pop(args));
	    break;
	case 2:
	    rb_iv_set(self, "@summary", rb_ary_pop(args));
	    break;
	case 3:
	    rb_iv_set(self, "@location", rb_ary_pop(args));
	    break;
	case 4:
	    rb_iv_set(self, "@description", rb_ary_pop(args));
	    break;
	}
    }

    return self;
}

/* reader function for @uid */
static VALUE ev_get_uid(VALUE self)
{
    return rb_iv_get(self, "@uid");
}

/* reader function for @start */
static VALUE ev_get_start_time(VALUE self)
{
    return rb_iv_get(self, "@start");
}

/* reader function for @duration */
static VALUE ev_get_duration(VALUE self)
{
    return rb_iv_get(self, "@duration");
}

/* reader function for @title */
static VALUE ev_get_title(VALUE self)
{
    return rb_iv_get(self, "@title");
}

/* reader function for @summary */
static VALUE ev_get_summary(VALUE self)
{
    return rb_iv_get(self, "@summary");
}

/* reader function for @location */
static VALUE ev_get_location(VALUE self)
{
    return rb_iv_get(self, "@location");
}

/* reader function for @description */
static VALUE ev_get_description(VALUE self)
{
    return rb_iv_get(self, "@description");
}

/* writer function for @uid */
static VALUE ev_set_uid(VALUE self, VALUE uid)
{
    return rb_iv_set(self, "@uid", uid);
}

/* writer function for @start */
static VALUE ev_set_start_time(VALUE self, VALUE start_time)
{
    return rb_iv_set(self, "@start", start_time);
}

/* writer function for @duration */
static VALUE ev_set_duration(VALUE self, VALUE duration)
{
    return rb_iv_set(self, "@duration", duration);
}

/* writer function for @title */
static VALUE ev_set_title(VALUE self, VALUE title)
{
    return rb_iv_set(self, "@title", title);
}

/* writer function for @summary */
static VALUE ev_set_summary(VALUE self, VALUE summary)
{
    return rb_iv_set(self, "@summary", summary);
}

/* writer function for @location */
static VALUE ev_set_location(VALUE self, VALUE location)
{
    return rb_iv_set(self, "@location", location);
}

/* writer function for @description */
static VALUE ev_set_description(VALUE self, VALUE description)
{
    return rb_iv_set(self, "@description", description);
}

/* initialize this class */
void Init_event()
{
    extern VALUE cCTime;

    cEvent = rb_define_class("Event", cCTime);

    rb_define_method(cEvent, "initialize", ev_init, -2);

    /* define some reader methods */
    rb_define_method(cEvent, "uid", ev_get_uid, 0);
    rb_define_method(cEvent, "start", ev_get_start_time, 0);
    rb_define_method(cEvent, "duration", ev_get_duration, 0);
    rb_define_method(cEvent, "summary", ev_get_summary, 0);
    rb_define_method(cEvent, "location", ev_get_location, 0);
    rb_define_method(cEvent, "description", ev_get_description, 0);

    /* define some writer methods */
    rb_define_method(cEvent, "uid=", ev_set_uid, 1);
    rb_define_method(cEvent, "start=", ev_set_start_time, 1);
    rb_define_method(cEvent, "duration=", ev_set_duration, 1);
    rb_define_method(cEvent, "summary=", ev_set_summary, 1);
    rb_define_method(cEvent, "location=", ev_set_location, 1);
    rb_define_method(cEvent, "description=", ev_set_description, 1);

    return;
}
